@@ -10,6 +10,7 @@ from evelink.parsing.orders import parse_market_orders
 from evelink.parsing.wallet_journal import parse_wallet_journal
 from evelink.parsing.wallet_transactions import parse_wallet_transactions
 
+
 class Corp(object):
     """Wrapper around /corp/ of the EVE API.
 
@@ -19,7 +20,8 @@ class Corp(object):
     def __init__(self, api):
         self.api = api
 
-    def corporation_sheet(self, corp_id=None):
+    @api.auto_call('corp/CorporationSheet', map_params={'corp_id': 'corporationID'})
+    def corporation_sheet(self, corp_id=None, api_result=None):
         """Get information about a corporation.
 
         NOTE: This method may be called with or without specifying
@@ -29,11 +31,6 @@ class Corp(object):
         a corp api key *must* be provided, and the private information
         for that corporation will be returned along with the public info.
         """
-        params = {}
-        if corp_id is not None:
-            params['corporationID'] = corp_id
-
-        api_result = self.api.get("corp/CorporationSheet", params)
 
         def get_logo_details(logo_result):
             _str, _int, _float, _bool, _ts = api.elem_getters(logo_result)
@@ -84,7 +81,7 @@ class Corp(object):
                 'wallets': 'walletDivisions',
             }
 
-            for key, rowset_name in division_types.iteritems():
+            for key, rowset_name in division_types.items():
                 divisions = {}
                 for row in rowsets[rowset_name].findall('row'):
                     a = row.attrib
@@ -94,20 +91,19 @@ class Corp(object):
 
         return api.APIResult(result, api_result.timestamp, api_result.expires)
 
-    def industry_jobs(self):
+    @api.auto_call('corp/IndustryJobs')
+    def industry_jobs(self, api_result=None):
         """Get a list of jobs for a corporation."""
-
-        api_result = self.api.get('corp/IndustryJobs')
-
         return api.APIResult(parse_industry_jobs(api_result.result), api_result.timestamp, api_result.expires)
 
-    def npc_standings(self):
+    @api.auto_call('corp/Standings')
+    def npc_standings(self, api_result=None):
         """Returns information about the corporation's standings towards NPCs.
 
         NOTE: This is *only* NPC standings. Player standings are accessed
         via the 'contacts' method.
         """
-        api_result = self.api.get('corp/Standings')
+
         container = api_result.result.find('corporationNPCStandings')
 
         rowsets = dict((r.attrib['name'], r) for r in container.findall('rowset'))
@@ -123,7 +119,7 @@ class Corp(object):
             'factions': 'factions',
         }
 
-        for key, rowset_name in _standing_types.iteritems():
+        for key, rowset_name in _standing_types.items():
             for row in rowsets[rowset_name].findall('row'):
                 a = row.attrib
                 standing = {
@@ -135,25 +131,19 @@ class Corp(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def kills(self, before_kill=None):
+    @api.auto_call('corp/KillLog', map_params={'before_kill': 'beforeKillID'})
+    def kills(self, before_kill=None, api_result=None):
         """Look up recent kills for a corporation.
 
         before_kill:
             Optional. Only show kills before this kill id. (Used for paging.)
         """
 
-        params = {}
-        if before_kill is not None:
-            params['beforeKillID'] = before_kill
-        api_result = self.api.get('corp/KillLog', params)
-
         return api.APIResult(parse_kills(api_result.result), api_result.timestamp, api_result.expires)
 
-    def wallet_info(self):
+    @api.auto_call('corp/AccountBalance')
+    def wallet_info(self, api_result=None):
         """Get information about corp wallets."""
-
-        api_result = self.api.get('corp/AccountBalance')
-
         rowset = api_result.result.find('rowset')
         results = {}
         for row in rowset.findall('row'):
@@ -166,37 +156,23 @@ class Corp(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def wallet_journal(self, before_id=None, limit=None):
+    @api.auto_call('corp/WalletJournal', map_params={'before_id': 'fromID', 'limit': 'rowCount', 'account': 'accountKey'})
+    def wallet_journal(self, before_id=None, limit=None, account=None, api_result=None):
         """Returns wallet journal for a corporation."""
-
-        params = {}
-        if before_id is not None:
-            params['fromID'] = before_id
-        if limit is not None:
-            params['rowCount'] = limit
-        api_result = self.api.get('corp/WalletJournal', params)
-
         return api.APIResult(parse_wallet_journal(api_result.result), api_result.timestamp, api_result.expires)
 
-    def wallet_transactions(self, before_id=None, limit=None):
+    @api.auto_call('corp/WalletTransactions', map_params={'before_id': 'fromID', 'limit': 'rowCount', 'account': 'accountKey'})
+    def wallet_transactions(self, before_id=None, limit=None, account=None, api_result=None):
         """Returns wallet transactions for a corporation."""
-
-        params = {}
-        if before_id is not None:
-            params['fromID'] = before_id
-        if limit is not None:
-            params['rowCount'] = limit
-        api_result = self.api.get('corp/WalletTransactions', params)
-
         return api.APIResult(parse_wallet_transactions(api_result.result), api_result.timestamp, api_result.expires)
 
-    def orders(self):
+    @api.auto_call('corp/MarketOrders')
+    def orders(self, api_result=None):
         """Return a corporation's buy and sell orders."""
-        api_result = self.api.get('corp/MarketOrders')
-
         return api.APIResult(parse_market_orders(api_result.result), api_result.timestamp, api_result.expires)
 
-    def assets(self):
+    @api.auto_call('corp/AssetList')
+    def assets(self, api_result=None):
         """Get information about corp assets.
 
         Each item is a dict, with keys 'id', 'item_type_id',
@@ -218,16 +194,16 @@ class Corp(object):
         the top-level values as "containers" with no fields except for
         "contents" and "location_id".
         """
-        api_result = self.api.get('corp/AssetList')
+
         return api.APIResult(parse_assets(api_result.result), api_result.timestamp, api_result.expires)
 
-    def faction_warfare_stats(self):
+    @api.auto_call('corp/FacWarStats')
+    def faction_warfare_stats(self, api_result=None):
         """Returns stats from faction warfare if this corp is enrolled.
 
         NOTE: This will raise an APIError if the corp is not enrolled in
         Faction Warfare.
         """
-        api_result = self.api.get('corp/FacWarStats')
 
         _str, _int, _float, _bool, _ts = api.elem_getters(api_result.result)
 
@@ -252,27 +228,24 @@ class Corp(object):
 
         return api.APIResult(result, api_result.timestamp, api_result.expires)
 
-    def contract_bids(self):
+    @api.auto_call('corp/ContractBids')
+    def contract_bids(self, api_result=None):
         """Lists the latest bids that have been made to any recent auctions."""
-        api_result = self.api.get('corp/ContractBids')
-
         return api.APIResult(parse_contract_bids(api_result.result), api_result.timestamp, api_result.expires)
 
-    def contract_items(self, contract_id):
+    @api.auto_call('corp/ContractItems', map_params={'contract_id': 'contractID'})
+    def contract_items(self, contract_id, api_result=None):
         """Lists items that a specified contract contains"""
-        api_result = self.api.get('corp/ContractItems', {'contractID': contract_id})
-
         return api.APIResult(parse_contract_items(api_result.result), api_result.timestamp, api_result.expires)
 
-    def contracts(self):
+    @api.auto_call('corp/Contracts')
+    def contracts(self, api_result=None):
         """Get information about corp contracts."""
-        api_result = self.api.get('corp/Contracts')
         return api.APIResult(parse_contracts(api_result.result), api_result.timestamp, api_result.expires)
 
-    def shareholders(self):
+    @api.auto_call('corp/Shareholders')
+    def shareholders(self, api_result=None):
         """Get information about a corp's shareholders."""
-        api_result = self.api.get('corp/Shareholders')
-
         results = {
             'char': {},
             'corp': {},
@@ -303,15 +276,14 @@ class Corp(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def contacts(self):
+    @api.auto_call('corp/ContactList')
+    def contacts(self, api_result=None):
         """Return the corp's corp and alliance contact lists."""
-        api_result = self.api.get('corp/ContactList')
         return api.APIResult(parse_contact_list(api_result.result), api_result.timestamp, api_result.expires)
 
-    def titles(self):
+    @api.auto_call('corp/Titles')
+    def titles(self, api_result=None):
         """Returns information about the corporation's titles."""
-        api_result = self.api.get('corp/Titles')
-
         rowset = api_result.result.find('rowset')
         results = {}
         for row in rowset.findall('row'):
@@ -336,11 +308,11 @@ class Corp(object):
                     roles[role['id']] = role
                 return roles
 
-            for key, rowset_name in constants.Corp.role_types.iteritems():
+            for key, rowset_name in constants.Corp.role_types.items():
                 roles = get_roles(rowset_name)
                 title['roles'][key] = roles
 
-            for key, rowset_name in constants.Corp.grantable_types.iteritems():
+            for key, rowset_name in constants.Corp.grantable_types.items():
                 roles = get_roles(rowset_name)
                 title['can_grant'][key] = roles
 
@@ -348,10 +320,9 @@ class Corp(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def starbases(self):
+    @api.auto_call('corp/StarbaseList')
+    def starbases(self, api_result=None):
         """Returns information about the corporation's POSes."""
-        api_result = self.api.get('corp/StarbaseList')
-
         rowset = api_result.result.find('rowset')
         results = {}
         for row in rowset.findall('row'):
@@ -370,10 +341,9 @@ class Corp(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def starbase_details(self, starbase_id):
+    @api.auto_call('corp/StarbaseDetail', map_params={'starbase_id': 'itemID'})
+    def starbase_details(self, starbase_id, api_result=None):
         """Returns details about the specified POS."""
-        api_result = self.api.get('corp/StarbaseDetail', {'itemID': starbase_id})
-
         _str, _int, _float, _bool, _ts = api.elem_getters(api_result.result)
 
         general_settings = api_result.result.find('generalSettings')
@@ -459,12 +429,13 @@ class Corp(object):
 
         return api.APIResult(result, api_result.timestamp, api_result.expires)
 
-    def members(self, extended=True):
+    def members(self, extended=True, api_result=None):
         """Returns details about each member of the corporation."""
-        args = {}
-        if extended:
-            args['extended'] = 1
-        api_result = self.api.get('corp/MemberTracking', args)
+        if api_result is None:
+            args = {}
+            if extended:
+                args['extended'] = 1
+            api_result = self.api.get('corp/MemberTracking', params=args)
 
         rowset = api_result.result.find('rowset')
         results = {}
@@ -505,10 +476,9 @@ class Corp(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def permissions(self):
+    @api.auto_call('corp/MemberSecurity')
+    def permissions(self, api_result=None):
         """Returns information about corporation member permissions."""
-        api_result = self.api.get('corp/MemberSecurity')
-
         results = {}
         rowset = api_result.result.find('rowset')
         for row in rowset.findall('row'):
@@ -527,7 +497,7 @@ class Corp(object):
 
             def get_roleset(roles_dict):
                 roles_group = {}
-                for key, rowset_name in roles_dict.iteritems():
+                for key, rowset_name in roles_dict.items():
                     roles = {}
                     roles_rowset = rowsets[rowset_name]
                     for role_row in roles_rowset.findall('row'):
@@ -543,11 +513,10 @@ class Corp(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def permissions_log(self):
+    @api.auto_call('corp/MemberSecurityLog')
+    def permissions_log(self, api_result=None):
         """Returns information about changes to member permissions."""
-        api_result = self.api.get('corp/MemberSecurityLog')
-
-        inverse_role_types = dict((v,k) for k,v in constants.Corp.role_types.iteritems())
+        inverse_role_types = dict((v,k) for k,v in constants.Corp.role_types.items())
 
         results = []
         rowset = api_result.result.find('rowset')
@@ -586,10 +555,9 @@ class Corp(object):
         results.sort(key=lambda r: r['timestamp'], reverse=True)
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def stations(self):
+    @api.auto_call('corp/OutpostList')
+    def stations(self, api_result=None):
         """Returns information about the corporation's (non-POS) stations."""
-        api_result = self.api.get('corp/OutpostList')
-
         rowset = api_result.result.find('rowset')
         results = {}
         for row in rowset.findall('row'):
@@ -612,10 +580,9 @@ class Corp(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def station_services(self, station_id):
+    @api.auto_call('corp/OutpostServiceDetail', map_params={'station_id': 'itemID'})
+    def station_services(self, station_id, api_result=None):
         """Returns information about a given station's services."""
-        api_result = self.api.get('corp/OutpostServiceDetail', {'itemID': station_id})
-
         rowset = api_result.result.find('rowset')
         results = {}
         for row in rowset.findall('row'):
@@ -633,10 +600,9 @@ class Corp(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def medals(self):
+    @api.auto_call('corp/Medals')
+    def medals(self, api_result=None):
         """Returns information about the medals created by a corporation."""
-        api_result = self.api.get('corp/Medals')
-
         rowset = api_result.result.find('rowset')
         results = {}
         for row in rowset.findall('row'):
@@ -652,10 +618,9 @@ class Corp(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def member_medals(self):
+    @api.auto_call('corp/MemberMedals')
+    def member_medals(self, api_result=None):
         """Returns information about medals assigned to corporation members."""
-        api_result = self.api.get("corp/MemberMedals")
-
         rowset = api_result.result.find('rowset')
         results = {}
         for row in rowset.findall('row'):
@@ -672,10 +637,9 @@ class Corp(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def container_log(self):
+    @api.auto_call('corp/ContainerLog')
+    def container_log(self, api_result=None):
         """Returns a log of actions performed on corporation containers."""
-        api_result = self.api.get("corp/ContainerLog")
-
         results = []
         rowset = api_result.result.find('rowset')
 
@@ -712,12 +676,8 @@ class Corp(object):
 
         return api.APIResult(results, api_result.timestamp, api_result.expires)
 
-    def locations(self, location_list):
-        params={
-            'IDs' : location_list,
-        }
-        api_result = self.api.get('corp/Locations', params)
-
+    @api.auto_call('corp/Locations', map_params={'location_list': 'IDs'})
+    def locations(self, location_list, api_result=None):
         rowset = api_result.result.find('rowset')
         rows = rowset.findall('row')
 
